import pydantic
from typing import Dict, Any, List

from odd_ge_adapter.domain import Expectation


class Suite(pydantic.BaseModel):
    expectation_suite_name: str
    expectations: List[Expectation]
    evaluation_parameters: Any



from odd_ge_adapter.utils.secret import hide_secret


class AccountIdError(Exception):
    def __init__(self, key, secret):
        super().__init__(self._message(key, secret))

    @staticmethod
    def _message(key: str, secret: str):
        return f"Could not get account id for {hide_secret(key)}, {hide_secret(secret)}.\nSet env variable aws_account_id explicitly or check your Account"



from datetime import datetime
import logging
from typing import List

from oddrn_generator.generators import S3Generator
from odd_models.models import (
    DataEntity,
    DataQualityTestRun,
    QualityRunStatus,
    DataEntityType,
    MetadataExtension,
)

from odd_ge_adapter.domain import RunResult, Result, RunResultMeta
from odd_ge_adapter.domain.s3_uri import S3Uri
from odd_ge_adapter.mappers.s3_uri_to_oddrn import s3_uri_to_oddrn
from odd_ge_adapter.oddrn import GreatExpectationGenerator


def map_result(
    run_result: RunResult,
    oddrn_generator: GreatExpectationGenerator,
    s3_oddrn_generator: S3Generator,
) -> List[DataEntity]:
    """
    Mapping through each validation result

    Args:
        run_result: odd_ge_adapter.domain.RunResult
        oddrn_generator: GreatExpectationGenerator
    Returns:
        List[DataEntity]
    """
    suite_name = run_result.meta.expectation_suite_name

    run_id = run_result.meta.run_id
    run_name = run_id.run_name
    run_time = run_id.run_time

    metadata = _map_metadata(run_result.meta, s3_oddrn_generator)

    oddrn_generator.set_oddrn_paths(suites=suite_name)

    return [
        _map_result(result, run_name, run_time, run_time, metadata, oddrn_generator)
        for result in run_result.results
    ]


def _map_result(
    result: Result,
    run_name: str,
    start_time: datetime,
    end_time: datetime,
    metadata: MetadataExtension,
    oddrn_generator: GreatExpectationGenerator,
) -> DataEntity:
    expectation_type = result.expectation_config.unique_type

    oddrn_generator.set_oddrn_paths(types=expectation_type, runs=run_name)

    qt_run = DataQualityTestRun(
        data_quality_test_oddrn=oddrn_generator.get_oddrn_by_path("types"),
        start_time=start_time,
        end_time=end_time,
        status=_map_status(result.success),
    )

    return DataEntity(
        oddrn=oddrn_generator.get_oddrn_by_path("runs"),
        name=f"{run_name}.{expectation_type}",
        type=DataEntityType.JOB_RUN,
        metadata=[metadata],
        data_quality_test_run=qt_run,
    )


def _map_status(success: bool):
    return QualityRunStatus.SUCCESS if success else QualityRunStatus.FAILED


def _map_metadata(
    run_meta: RunResultMeta, s3_oddrn_generator: S3Generator
) -> MetadataExtension:
    run_id = run_meta.run_id

    try:
        uris = run_meta.batch_kwargs.get("odd_metadata")
        datasets = [s3_uri_to_oddrn(S3Uri(uri), s3_oddrn_generator) for uri in uris]
    except KeyError:
        datasets = []
        logging.exception("Could not get dataset name")

    meta = {
        "version": run_meta.great_expectations_version,
        "expectation_suite_name": run_meta.expectation_suite_name,
        "run_name": run_id.run_name,
        "run_time": run_id.run_time,
        "dataset": datasets,
    }

    return MetadataExtension(
        schema_url="https://raw.githubusercontent.com/opendatadiscovery/opendatadiscovery-specification/main/specification/extensions/great_expectations.json#/definitions/GreatExpectationsMetadata",
        metadata=meta,
    )



from typing import Dict, Any, List
from datetime import datetime
import pydantic

from odd_ge_adapter.domain import Expectation


class RunIdentifier(pydantic.BaseModel):
    run_name: str
    run_time: datetime


class RunResultMeta(pydantic.BaseModel):
    batch_kwargs: Dict[Any, Any]
    expectation_suite_name: str
    great_expectations_version: str
    run_id: RunIdentifier


class ExceptionInfo(pydantic.BaseModel):
    exception_message: str
    exception_traceback: str
    raised_exception: bool


class Result(pydantic.BaseModel):
    expectation_config: Expectation
    meta: Dict[Any, Any]
    result: Dict[Any, Any]
    success: bool


class RunResult(pydantic.BaseModel):
    meta: RunResultMeta
    results: List[Result]



from typing import Any, Dict

import pydantic


class Expectation(pydantic.BaseModel):
    expectation_type: str
    kwargs: Dict[str, Any]
    meta: Dict[str, Any]

    @property
    def unique_type(self):
        """
        Returns
        self.expectation_type -> for common expectations
        self.expectation_type + column_name -> for columns expectations, because need to generate unique oddrn
        """

        if self.expectation_type.startswith("expect_column"):
            return f"{self.expectation_type}:{self.kwargs.get('column')}"

        return self.expectation_type



import logging
from typing import List

from odd_models.models import (
    DataEntity,
    DataQualityTest,
    DataQualityTestExpectation,
    DataEntityType,
)
from oddrn_generator.generators import S3Generator

from odd_ge_adapter.domain import Suite, Expectation, S3Uri
from odd_ge_adapter.mappers.s3_uri_to_oddrn import s3_uri_to_oddrn
from odd_ge_adapter.oddrn import GreatExpectationGenerator


def _dataset_list_to_oddrn(
    dataset_list: List[str], oddrn_gen: S3Generator
) -> List[str]:
    s3_uris = [S3Uri(dataset) for dataset in dataset_list]
    return [s3_uri_to_oddrn(s3_uri, oddrn_gen) for s3_uri in s3_uris]


def _get_dataset_list(suite: Suite, oddrn: S3Generator) -> List[str]:
    """
    Try to get dataset list. Wait that dataset list set in:  evaluation_parametres -> odd_metadata

    Examples:
        ['s3://bucket/folder','s3://bucket/folder/sub_folder']
    """

    try:
        datasets = suite.evaluation_parameters.get("odd_metadata")
        return _dataset_list_to_oddrn(datasets, oddrn)
    except KeyError as e:
        logging.exception("Couldn't take meta data")


def _map_expectation(
    expectation: Expectation,
    dataset_list: List[str],
    oddrn_generator: GreatExpectationGenerator,
) -> DataEntity:
    """
    Map expectation to DataEntity

    Args:
        expectation: odd_ge_adapter.domain.Expectation
        oddrn_generator: GreatExpectationGenerator

    """

    suite_name = oddrn_generator.get_oddrn_by_path("suites")

    expectation_type = expectation.unique_type
    qt = DataQualityTest(
        suite_name=suite_name,
        expectation=DataQualityTestExpectation(
            type=expectation_type,
            additionalProperties=str(expectation.kwargs),
        ),
        dataset_list=dataset_list,
    )

    oddrn_generator.set_oddrn_paths(types=expectation_type)
    return DataEntity(
        oddrn=oddrn_generator.get_oddrn_by_path("types"),
        name=f"{suite_name}.{expectation_type}",
        type=DataEntityType.JOB,
        metadata=[],
        data_quality_test=qt,
    )


def map_suite(
    suite: Suite,
    oddrn_generator: GreatExpectationGenerator,
    s3_oddrn_generator: S3Generator,
) -> List[DataEntity]:
    """
    Returns list of mapped odd_ge_adapter.domain.Expectation.
    Treat each expectation as is an individual entity with oddrn

    Args:
        suite: odd_ge_adapter.domain.Suite
        oddrn_generator: GreatExpectationGenerator
        s3_oddrn_generator: S3Generator - needs for cast datasets to oddrn
    """

    dataset_list = _get_dataset_list(suite, s3_oddrn_generator)

    oddrn_generator.set_oddrn_paths(suites=suite.expectation_suite_name)

    return [
        _map_expectation(exp, dataset_list, oddrn_generator)
        for exp in suite.expectations
    ]



"""
    dataset_list: [
        s3://bucket/datasets,
        s3://bucket/datasets/subfolder
    ]
"""
from urllib.parse import urlparse


class S3Uri:
    def __init__(self, path: str, join_key: str = ":"):
        self._parsed = urlparse(path)
        self._join_key = join_key

    @property
    def schema(self):
        return self._parsed.scheme

    @property
    def bucket(self):
        return self._parsed.netloc

    @property
    def path(self):
        """"""
        return self._parsed.path.lstrip("/").replace("/", self._join_key)



from typing import Optional, Union
from typing_extensions import Literal

import pydantic
from odd_collector_sdk.domain.plugin import Plugin
from pydantic import SecretStr
from typing_extensions import Annotated


class LocalStoragePlugin(Plugin):
    type: Literal["local"]
    suites_path: pydantic.FilePath
    results_path: pydantic.FilePath


class S3StoragePlugin(Plugin):
    type: Literal["s3"]
    bucket: str
    results_key: str
    suites_key: str
    aws_access_key_id: SecretStr
    aws_secret_access_key: SecretStr
    aws_region: str
    aws_account_id: Optional[str]
    aws_session_token: Optional[SecretStr]


AvailablePlugin = Annotated[
    Union[S3StoragePlugin, LocalStoragePlugin],
    pydantic.Field(discriminator="type"),
]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['odd_ge_adapter',
 'odd_ge_adapter.adapters',
 'odd_ge_adapter.adapters.s3',
 'odd_ge_adapter.domain',
 'odd_ge_adapter.mappers',
 'odd_ge_adapter.storage',
 'odd_ge_adapter.utils']

package_data = \
{'': ['*']}

install_requires = \
['black>=22.3.0,<23.0.0',
 'boto3==1.21.21',
 'odd-collector-sdk==0.2.6',
 'oddrn-generator>=0.1.37,<0.2.0',
 'pytest>=7.1.2,<8.0.0',
 's3>=3.0.0,<4.0.0']

setup_kwargs = {
    'name': 'odd-ge-adapter',
    'version': '0.2.13',
    'description': 'ODD adapter to Great Expectations',
    'long_description': '## ODD GreatExpectation adapter\n\nLibrary for GreatExpectation schema compatible json files for suites and validation, reads them from s3 storage and ingests to OpenDataDiscovery platform.\n\nSuites must include special metadata about folder with data which will be validated in the future.\nExample \n```json\n{\n  "expectation_suite_name": "notification",\n  ...\n  "meta": {\n    "batch_kwargs": {\n      ...\n      "odd_metadata": [\n        "s3://bucket/folder"\n      ]\n    }\n  }\n}\n```\n\nValidation results must include metadata about which file was validated.\nExample\n```json\n{\n  "meta": {\n    "batch_kwargs": {\n      ...\n      "odd_metadata": [\n        "s3://bucket/folder/file.csv"\n      ]\n    },\n    ...\n    "validation_time": "20220525T174015.717924Z"\n  },\n  "results": [...],\n  "success": true\n}\n```\n\n### Library usage\nInstall package\n```bash\npip install odd-ge-adapter\n```\n\nExample:\n```\nfrom odd_models.api_client import ODDApiClient\nfrom odd_ge_adapter.domain.plugin import S3StoragePlugin\nfrom odd_ge_adapter.adapters.s3.adapter import Adapter\n\nplatform = ODDApiClient(<PLATFORM_URL>)\n\n# aws credentials can be omitted and taken from the ENV variables \nconfig = S3StoragePlugin(\n    type="s3",\n    name="s3_storage",\n    bucket="",\n    results_key="",\n    suites_key="",\n    # aws_account_id="",\n    # aws_region="",\n    # aws_access_key_id="",\n    # aws_secret_access_key="",\n    # aws_session_token=""\n)\n\nadapter = Adapter(config)\nplatform.post_data_entity_list(data=adapter.get_data_entity_list())\n```\n\n\n### As collector usage\nIn progress...\n\n',
    'author': 'Open Data Discovery',
    'author_email': 'pypi@opendatadiscovery.org',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/opendatadiscovery/odd-ge-adapter',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
